
Local playwhat$ = "wayout1aX1f.it2145.it"

AppTitle "BassMod Player"

Graphics 800,600 , 0,2  ; larger window size
Const sw=640,sh=480
Graphics sw,sh , 0,3  ; smaller (scaled) graphics content
SetBuffer BackBuffer()
Global r1=192,g1=224,b1=255  ; inner text color (body)
Global r2=0,g2=0,b2=0  ; outer text color (border)
Const r3=0,g3=64,b3=128  ; background color
ClsColor r3,g3,b3
Cls
SetFont LoadFont( "Lucida Console" , 20 )

If CommandLine() <> ""
    playwhat = CommandLine()
End If

Global spec$ = playwhat
If FileType(spec) <> 1
    RuntimeError "Unable to find music file:"+Chr(13)+spec
End If

Include "TimerSystem 4c.BB"

Include "bassmod.bb"
If Not BASSMOD_Init( -1 , 44100 , 0 )
    RuntimeError "Unable to initialize BASSMOD:"+Chr(13)+BASSMOD_GetErrorString$()
EndIf

; loop the music endlessly, and smoothen sharp sample cuts
Const playflags = BASS_MUSIC_LOOP + BASS_MUSIC_RAMPS + BASS_MUSIC_STOPBACK + BASS_MUSIC_POSRESET
Const loadflags = playflags + BASS_MUSIC_CALCLEN
Local music = BASSMOD_MusicLoad( 0 , spec , 0,0 , flags )
If Not music
    RuntimeError "Unable to load music file:"+Chr(13)+BASSMOD_GetErrorString$()
End If

Local musicname$ = BASSMOD_MusicGetName()

Local playlen = BASSMOD_MusicGetLength( 1 )
Local ordmax = BASSMOD_MusicGetLength( 0 )

; perhaps use only the amplification for independently controlling the music volume
; although, depending on the music, high values (85+) can cause distortion
Local amp = 75  ; default = 50 (min=0,max=100)
BASSMOD_MusicSetAmplify amp

; bassmod's set/get volume seems to mess with Windows' "wave" volume
; which overrides a user's preference (!) and leaves it altered after quitting (!)
Local vol = BASSMOD_GetVolume()  ; for Windows i usually recommended a default of 100
BASSMOD_SetVolume vol

Local pansep = 50  ; default = 50 (min=0,max=100)
BASSMOD_MusicSetPanSep pansep

BASSMOD_MusicPlay

MilliSecs = MilliSecs()
Cls
Flip

; wait for the player to begin (perfect sync!)
Local timeout_timer = NewTimer( 3333 )
Repeat
    Delay 5
    MilliSecs = MilliSecs()
    If TimerDone( timeout_timer )
        RuntimeError "Unable to start the music"
    End If
Until BASSMOD_MusicIsActive() = BASS_ACTIVE_PLAYING

Const check_oldrow = 63
Const check_currow = 0
Local ordtime = 0
Local musictimer = NewTimer( infinity )  ; keep track of music time

Local fadetimer = NewTimer( 2000 )  ; duration of fade out
Local fade  ; currently fading out
Local startfadevol  ; current amplification level for fade out
Local quit  ; quit mainloop instantly

.replay
Local oldrow = check_oldrow
Local currow = check_currow
Local ordstart = MilliSecs

Const sx=20,sy=16  ; spacing per char
Const tx=sx/1.5,ty=sy/1.5  ; text position
Repeat
    MilliSecs = MilliSecs()
    
    ; debug: track order duration using rows
    ; - perhaps best way is to sync image transition to rows instead of time
	Local order = music_order()
	Local row = music_row()
    Local rowchanged = False
    If row <> currow
        oldrow = currow
        currow = row
        If currow = check_currow And oldrow = check_oldrow
            ordtime = MilliSecs - ordstart
            ordstart = MilliSecs
        End If
		rowchanged = True
    End If
    
    ; music control
    If KeyDown(Key_Up) Or KeyDown(Key_Down)
        vol = vol + 1 * ( KeyDown(Key_Up) - KeyDown(Key_Down) )
        If vol > 100 Then vol = 100
        If vol < 0 Then vol = 0
        BASSMOD_SetVolume vol
    End If
    If fade
        amp = (1-TimerDelta(fadetimer))*startfadevol
        If amp < 0 Then amp = 0
        If amp > 100 Then amp = 100
        If amp <= 0
            quit = True
            BASSMOD_MusicSetAmplify 0
        Else
            BASSMOD_MusicSetAmplify amp
        End If
    Else
        If KeyDown(Key_Left) Or KeyDown(Key_Right)
            amp = amp + 1 * ( KeyDown(Key_Right) - KeyDown(Key_Left) )
            If amp > 100 Then amp = 100
            If amp < 0 Then amp = 0
            BASSMOD_MusicSetAmplify amp
        End If
    End If
    If KeyHit(Key_Space)
        Select BASSMOD_MusicIsActive()
            Case BASS_ACTIVE_PLAYING
                BASSMOD_MusicPause
                PauseTimer musictimer
            Case BASS_ACTIVE_PAUSED
                BASSMOD_MusicPlay
                ResumeTimer musictimer
        End Select
    End If
	If KeyHit(Key_Enter)
		Local neword
		If music_order() = ordmax-1
			neword = 0
		Else
			neword = music_order() + 1
		End If
		Local pos = BASSMOD_MakeLong( neword , 0 )
		DebugLog "skipto: "+BASSMOD_MusicPlayEx( pos , playflags , 1 )
		Goto replay
	End If
    If KeyHit(1)  ; esc
        If Not fade
            fade = True
            startfadevol = amp
            ResetTimer fadetimer
        End If
    End If
    
    ; text display
	textout 0 , "File: "+playwhat
	textout 1 , "Title: "+musicname
	textout 3 , "<Space>      Playing:  "+BASSMOD_MusicIsActive()
	textout 4 , "<Enter>      Next order ("+(order+1)+" of "+ordmax+")"
	textout 6 , "<Up/Down>    Volume:   "+BASSMOD_GetVolume()+"%"
	textout 7 , "<Left/Right> Amplify:  "+amp+"%"
    If fade
        textoutR 7 , "Fading out"
    End If
	textout 9 , "             Position:  Order "+order+"  Row "+row
	textout 10, "             Duration: "+TimerElapsed(musictimer)
	textout 11, "             Order duration: "+ordtime+" ms"
	; extrapolate order time (estimated)
	; - assumes number of rows in pattern = 64 (!)
	Local maxrows = 64
	Local ordtime_ep
	If row > 0
		If rowchanged
            ordtime_ep = Float (MilliSecs-ordstart) * maxrows / row
		End If
	Else
		ordtime_ep = 0
	End If
    textout 12, "             Estimated order duration: "+ordtime_ep+" ms"
	
    Flip
    Cls
	Delay 2
    
Until quit

BASSMOD_MusicStop
BASSMOD_Free
End

Const infinity = 2147483647

Function music_order()
    Return BASSMOD_MusicGetPosition() And $0000FFFF
End Function
Function music_row()
    Return (BASSMOD_MusicGetPosition() And $FFFF0000) Shr 16
End Function

Function textout( y_mul , t$ )
	Color r1,g1,b1
	Text tx,ty+y_mul*sy , t
End Function
Function textoutR( y_mul , t$ )
	Color r1,g1,b1
	Text sw-tx-StringWidth(t),ty+y_mul*sy , t
End Function

;-------------------------------------------------------------------------------
Const Key_CursorUp%    = 200
Const Key_CursorDown%  = 208
Const Key_CursorLeft%  = 203
Const Key_CursorRight% = 205
Const Key_ArrowUp%    = Key_CursorUp
Const Key_ArrowDown%  = Key_CursorDown
Const Key_ArrowLeft%  = Key_CursorLeft
Const Key_ArrowRight% = Key_CursorRight
Const Key_Up%    = Key_ArrowUp
Const Key_Down%  = Key_ArrowDown
Const Key_Left%  = Key_ArrowLeft
Const Key_Right% = Key_ArrowRight
Const Key_Space%        =  57
Const Key_Enter%        =  28

;-------------------------------------------------------------------------------
; contact: Teasy <TheChange@yahoo.com>

;~IDEal Editor Parameters:
;~L#x
;~C#Blitz3D1105